@@ -10,7 +10,7 @@ from django.conf import settings
 from django.contrib.auth import (authenticate, logout, views)
 from django.contrib.auth.models import User
 from django.core.paginator import Paginator, InvalidPage, EmptyPage
-from django.db.models import Q
+from django.db.models import Q, Count
 from django.forms.formsets import formset_factory
 from django.http import HttpResponseRedirect, Http404, HttpResponse
 from django.shortcuts import render,render_to_response
@@ -32,21 +32,22 @@ def home_view(request):
     activity = []
     if request.user.is_authenticated():
         startdate = datetime.datetime.now()
-        staff = User.objects.filter(is_staff=True)
         for i in range(31,-1,-1):
             temp = startdate - datetime.timedelta(days=i)
             day = temp.strftime("%d")
             month = temp.strftime("%m")
             year = temp.strftime("%y")
-            count = Tracker.objects.filter(course__isnull=False, tracker_date__day=day,tracker_date__month=month,tracker_date__year=year).exclude(user_id__in=staff).count()
+            count = Tracker.objects.filter(course__isnull=False, course__is_draft=False, course__is_archived=False,tracker_date__day=day,tracker_date__month=month,tracker_date__year=year).count()
             activity.append([temp.strftime("%d %b %y"),count])
     leaderboard = Points.get_leaderboard(10)
     return render_to_response('oppia/home.html',{'recent_activity':activity, 'leaderboard':leaderboard}, context_instance=RequestContext(request))
 
 def course_view(request):
-    course_list = Course.objects.all().order_by('title')   
+    if request.user.is_staff:
+        course_list = Course.objects.all().order_by('title')
+    else:
+        course_list = Course.objects.filter(is_draft=False,is_archived=False).order_by('title')   
     startdate = datetime.datetime.now()
-    staff = User.objects.filter(is_staff=True)
     for course in course_list:
         course.activity = []
         for i in range(7,-1,-1):
@@ -54,7 +55,8 @@ def course_view(request):
             day = temp.strftime("%d")
             month = temp.strftime("%m")
             year = temp.strftime("%y")
-            count = Tracker.objects.filter(course = course, tracker_date__day=day,tracker_date__month=month,tracker_date__year=year).exclude(user_id__in=staff).count()
+            
+            count = Tracker.objects.filter(course = course, tracker_date__day=day,tracker_date__month=month,tracker_date__year=year).count()
             course.activity.append([temp.strftime("%d %b %y"),count])   
     return render_to_response('oppia/course/course.html',{'course_list': course_list,}, context_instance=RequestContext(request))
 
@@ -103,27 +105,28 @@ def upload(request):
     return render(request, 'oppia/upload.html', {'form': form,})
 
 def recent_activity(request,id):
-    course = Course.objects.get(pk=id)
+    course = check_can_view(request, id)
     dates = []
     startdate = datetime.datetime.now()
-    staff = User.objects.filter(is_staff=True)
     for i in range(31,-1,-1):
         temp = startdate - datetime.timedelta(days=i)
         day = temp.strftime("%d")
         month = temp.strftime("%m")
         year = temp.strftime("%y")
-        count_act_page = Tracker.objects.filter(course=course,type='page',tracker_date__day=day,tracker_date__month=month,tracker_date__year=year).exclude(user_id__in=staff).count()
-        count_act_quiz = Tracker.objects.filter(course=course,type='quiz',tracker_date__day=day,tracker_date__month=month,tracker_date__year=year).exclude(user_id__in=staff).count()
-        count_media = Tracker.objects.filter(course=course,type='media',tracker_date__day=day,tracker_date__month=month,tracker_date__year=year).exclude(user_id__in=staff).count()
-        dates.append([temp.strftime("%d %b %y"),count_act_page,count_act_quiz,count_media])
+        count_objs = Tracker.objects.filter(course=course,tracker_date__day=day,tracker_date__month=month,tracker_date__year=year).values('type').annotate(total=Count('type'))
+        count_activity = {'page':0, 'quiz':0, 'media':0, 'resource':0, 'total':0}
+        for co in count_objs:
+            count_activity[co['type']] = count_activity[co['type']] + co['total']
+            count_activity['total'] = count_activity['total'] + co['total']
+        
+        dates.append([temp.strftime("%d %b %y"),count_activity])
     leaderboard = Points.get_leaderboard(10, course)
     return render_to_response('oppia/course/activity.html',{'course': course,'data':dates, 'leaderboard':leaderboard}, context_instance=RequestContext(request))
 
 def recent_activity_detail(request,id):
     course = check_owner(request,id)
         
-    staff = User.objects.filter(is_staff=True)
-    trackers = Tracker.objects.filter(course=course).exclude(user_id__in=staff).order_by('-tracker_date')
+    trackers = Tracker.objects.filter(course=course).order_by('-tracker_date')
     paginator = Paginator(trackers, 25)
     # Make sure page request is an int. If not, deliver first page.
     try:
@@ -387,6 +390,17 @@ def check_owner(request,id):
         raise Http404
     return course
 
+def check_can_view(request,id):
+    try:
+        # check only the owner can view 
+        if request.user.is_staff:
+            course = Course.objects.get(pk=id)
+        else:
+            course = Course.objects.get(pk=id,is_draft=False,is_archived=False)
+    except Course.DoesNotExist:
+        raise Http404
+    return course
+
 def leaderboard_view(request):
     lb = Points.get_leaderboard(100)
     paginator = Paginator(lb, 25) # Show 25 contacts per page
